@@ -54,6 +54,7 @@
 
 #define ACTION_CONFIG_CIRCLE_INPUT_MODE_KEY       "CircleInputModeKey"
 #define ACTION_CONFIG_CIRCLE_KANA_MODE_KEY        "CircleKanaModeKey"
+#define ACTION_CONFIG_CIRCLE_LATIN_HIRAGANA_MODE_KEY  "CircleLatinHiraganaModeKey"
 #define ACTION_CONFIG_CIRCLE_TYPING_METHOD_KEY    "CircleTypingMethodKey"
 #define ACTION_CONFIG_LATIN_MODE_KEY              "LatinModeKey"
 #define ACTION_CONFIG_WIDE_LATIN_MODE_KEY         "WideLatinModeKey"
@@ -124,13 +125,6 @@
 #define ACTION_CONFIG_DICT_ADMIN_KEY              "DictAdminKey"
 #define ACTION_CONFIG_ADD_WORD_KEY                "AddWordKey"
 
-#define UTF8_BRACKET_CORNER_BEGIN "\xE3\x80\x8C"
-#define UTF8_BRACKET_CORNER_END   "\xE3\x80\x8D"
-#define UTF8_BRACKET_WIDE_BEGIN   "\xEF\xBC\xBB"
-#define UTF8_BRACKET_WIDE_END     "\xEF\xBC\xBD"
-#define UTF8_MIDDLE_DOT           "\xE3\x83\xBB"
-#define UTF8_SLASH_WIDE           "\xEF\xBC\x8F"
-
 #define N_(x) (x)
 
 AnthyInstance::AnthyInstance (FcitxInstance* instance) :
@@ -756,14 +750,17 @@ void AnthyInstance::set_symbol_style(SymbolStyle symbol)
     switch (m_config.m_symbol_style)
     {
         case FCITX_ANTHY_SYMBOL_STYLE_WIDEBRACKET_WIDESLASH:
-            m_preedit.set_bracket_style (FCITX_ANTHY_BRACKET_WIDE);
-            m_preedit.set_slash_style   (FCITX_ANTHY_SLASH_WIDE);
-        case FCITX_ANTHY_SYMBOL_STYLE_CORNERBRACKET_WIDESLASH:
             m_preedit.set_bracket_style (FCITX_ANTHY_BRACKET_JAPANESE);
             m_preedit.set_slash_style   (FCITX_ANTHY_SLASH_WIDE);
-        case FCITX_ANTHY_SYMBOL_STYLE_WIDEBRACKET_MIDDLEDOT:
+            break;
+        case FCITX_ANTHY_SYMBOL_STYLE_CORNERBRACKET_WIDESLASH:
+            m_preedit.set_bracket_style (FCITX_ANTHY_BRACKET_WIDE);
+            m_preedit.set_slash_style   (FCITX_ANTHY_SLASH_WIDE);
+            break;
+        case FCITX_ANTHY_SYMBOL_STYLE_CORNERBRACKET_MIDDLEDOT:
             m_preedit.set_bracket_style (FCITX_ANTHY_BRACKET_WIDE);
             m_preedit.set_slash_style   (FCITX_ANTHY_SLASH_JAPANESE);
+            break;
         case FCITX_ANTHY_SYMBOL_STYLE_JAPANESE:
         default:
             m_preedit.set_bracket_style (FCITX_ANTHY_BRACKET_JAPANESE);
@@ -1704,6 +1701,25 @@ AnthyInstance::action_circle_kana_mode (void)
     return true;
 }
 
+bool
+AnthyInstance::action_circle_latin_hiragana_mode (void)
+{
+    InputMode mode = get_input_mode ();
+
+    if (mode == FCITX_ANTHY_MODE_LATIN)
+    {
+        mode = FCITX_ANTHY_MODE_HIRAGANA;
+    } else if (mode == FCITX_ANTHY_MODE_HIRAGANA)
+    {
+        mode = FCITX_ANTHY_MODE_LATIN;
+    }
+
+    set_input_mode (mode);
+    save_config();
+
+    return true;
+}
+
 bool
 AnthyInstance::action_latin_mode (void)
 {
@@ -2107,6 +2123,7 @@ CONFIG_BINDING_REGISTER("KeyProfile", "CustomNicolaTable", m_nicola_fundamental_
 
 CONFIG_BINDING_REGISTER("Key", ACTION_CONFIG_CIRCLE_INPUT_MODE_KEY, m_key_default.m_hk_CIRCLE_INPUT_MODE)
 CONFIG_BINDING_REGISTER("Key", ACTION_CONFIG_CIRCLE_KANA_MODE_KEY, m_key_default.m_hk_CIRCLE_KANA_MODE)
+CONFIG_BINDING_REGISTER("Key", ACTION_CONFIG_CIRCLE_LATIN_HIRAGANA_MODE_KEY, m_key_default.m_hk_CIRCLE_LATIN_HIRAGANA_MODE)
 CONFIG_BINDING_REGISTER("Key", ACTION_CONFIG_CIRCLE_TYPING_METHOD_KEY, m_key_default.m_hk_CIRCLE_TYPING_METHOD)
 CONFIG_BINDING_REGISTER("Key", ACTION_CONFIG_LATIN_MODE_KEY, m_key_default.m_hk_LATIN_MODE)
 CONFIG_BINDING_REGISTER("Key", ACTION_CONFIG_WIDE_LATIN_MODE_KEY, m_key_default.m_hk_WIDE_LATIN_MODE)
@@ -2221,6 +2238,7 @@ std::string AnthyInstance::get_kana_table()
         "101kana.sty",
         "tsuki-2-203-101.sty",
         "tsuki-2-203-106.sty",
+        "qkana.sty",
         m_config.m_kana_fundamental_table
     };
 
@@ -2370,6 +2388,7 @@ void AnthyInstance::configure()
     // mode keys
     APPEND_ACTION (CIRCLE_INPUT_MODE,       action_circle_input_mode);
     APPEND_ACTION (CIRCLE_KANA_MODE,        action_circle_kana_mode);
+    APPEND_ACTION (CIRCLE_LATIN_HIRAGANA_MODE,action_circle_latin_hiragana_mode);
     APPEND_ACTION (CIRCLE_TYPING_METHOD,    action_circle_typing_method);
     APPEND_ACTION (LATIN_MODE,              action_latin_mode);
     APPEND_ACTION (WIDE_LATIN_MODE,         action_wide_latin_mode);
